import Tokenizer from './infra/tokenizer.js';
import TunaErrorHandler from "./infra/tunaErrorHandler.js";
import Configs from "./config.js";

export class Essentials {

  constructor(sessionID, env = "production") {
    if (!sessionID)
      throw new TunaErrorHandler("ERR:34");
    this.env = env;
    this.sessionID = sessionID;
    this.isTunaReady = true;
  }

  ping(){
    return `pong ${Configs[this.env].INTEGRATIONS_API_URL}`;
  }

  getSessionID() {
    return this.sessionID;
  }

  isTunaReady() {
    return this.isTunaReady;
  }

  maskCreditCard(creditCardNumber) {
    // clear formatting mask
    creditCardNumber = creditCardNumber.replace(/[^\da-zA-Z]/g, '');

    let maskedCreditCard = creditCardNumber.substring(0, 6) + "xxxxxx" + creditCardNumber.slice(-4);
    return maskedCreditCard;
  }

  tokenizator() {
    if (this.isTunaReady) {
      if (!this.tokenizer)
        this.tokenizer = new Tokenizer(this.sessionID);
      return this.tokenizer;
    }
    else
      throw new TunaErrorHandler("ERR:01");
  }

  async tokenize(creditCardData, resultCallback) {

    let isResponseByCallback = resultCallback && typeof resultCallback === "function";

    try {

      let tokenizator = this.tokenizator();

      //Create a safe copy of the credit card data object
      let safeCreditCardData = { ...creditCardData };
      safeCreditCardData.maskedNumber = this.maskCreditCard(safeCreditCardData.cardNumber);
      delete safeCreditCardData["cvv"];
      delete safeCreditCardData["cardNumber"];

      if (isResponseByCallback) {
        resultCallback(tokenizator.generate(creditCardData), safeCreditCardData);
      } else {

        let tokenResponse = await tokenizator.generate(creditCardData);

        if (tokenResponse && tokenResponse.code)
          return { success: tokenResponse.code === 1, tokenData: tokenResponse, cardData: safeCreditCardData };
        else
          return { success: false, tokenData: tokenResponse, cardData: safeCreditCardData };
      }

    } catch (e) {
      if (isResponseByCallback)
        resultCallback(new Promise((resolve, reject) => reject(e)), null);
      else
        return { success: false, message: e.message }
    }
  }

  async callPaymentEndpoint(body) {
    try {
      const options = {
        method: 'post',
        headers: {
          'Content-type': 'application/json; charset=UTF-8',
          'x-tuna-token-session': this.sessionID,
        },
        body: JSON.stringify(body)
      }

      return await (await fetch(`${Configs[this.env].TOKEN_API_URL.INTEGRATIONS_API_URL}/Init`, options)).json();

    } catch (error) {
      throw new TunaErrorHandler(error);
    }
  }

  async pay(paymentMethodType, totalPaymentAmount, paymentMethodAmount, preparedCardData) {

    let paymentMethod = {
      Amount: paymentMethodAmount
    };

    switch (paymentMethodType) {
      case "creditCard":
        let fullCardInfo;

        if (preparedCardData.alreadyTokenized)
          fullCardInfo = preparedCardData.fullCardInfo;
        else
          fullCardInfo = await this.tokenize(preparedCardData.cardData);

        if (fullCardInfo && (preparedCardData.alreadyTokenized || fullCardInfo.success === true)) {
          paymentMethod.PaymentMethodType = '1';
          paymentMethod.Installments = fullCardInfo.cardData.installment || 1;
          paymentMethod.CardInfo = {
            Token: fullCardInfo.tokenData.token,
            BrandName: fullCardInfo.tokenData.brand,
            SaveCard: !fullCardInfo.cardData.singleUse,
            ExpirationMonth: fullCardInfo.cardData.expirationMonth,
            ExpirationYear: fullCardInfo.cardData.expirationYear,
            CardHolderName: fullCardInfo.cardData.cardHolderName,
            CardNumber: fullCardInfo.cardData.maskedNumber.trim()
          };

        } else {
          throw new TunaErrorHandler("ERR:20");
        }
        break;
      case "pix":
        paymentMethod.PaymentMethodType = 'D';
        break;
      case "crypto":
        paymentMethod.PaymentMethodType = 'E';
        break;
      case "bankInvoice":
        paymentMethod.PaymentMethodType = '3';
        break;
    }

    let initRequest = {
      TokenSession: this.getSessionID(),
      PaymentData: {
        Amount: totalPaymentAmount,
        CountryCode: "BR",
        PaymentMethods: [paymentMethod]
      },
    };
    return await this.callPaymentEndpoint(initRequest);
  }

  async doStatusLongPolling(callbackFunction, methodID, paymentKey) {
    try {
      const options = {
        method: 'post',
        headers: {
          'Content-type': 'application/json; charset=UTF-8',
          'x-tuna-token-session': this.getSessionID()
        },
        body: JSON.stringify({ MethodID: methodID, PaymentStatusList: ['2', '8'], PaymentKey: paymentKey })
      }

      let response = await (await fetch(`${Configs[this.env].INTEGRATIONS_API_URL}/StatusPoll`, options)).json();
      if (response.paymentMethodConfimed)
        callbackFunction(response);
      else if (response.allowRetry)
        this.doStatusLongPolling(callbackFunction, methodID, paymentKey);
      else
        callbackFunction(response);

    } catch (error) {
      callbackFunction({ code: -1, paymentMethodConfimed: false, allowRetry: false }, error);
    }
  }

}

export default function factory(sessionID, env = "production") {
  if (sessionID)
    return new Essentials(sessionID, env);
  else
    throw new TunaErrorHandler("ERR:34");
}