import TunaErrorHandler from "./tunaErrorHandler.js";
import cardValidator from "card-validator";
import configs from "../config.js";


export default class Tokenizer {

    constructor(sessionID, env) {
        this.tokenAPIURL = configs[env].TOKEN_API_URL;
        this.sessionID = sessionID;
    }


    async bind(token, cvv) {
        try {
            const options = {
                method: 'post',
                headers: {
                    'Content-type': 'application/json; charset=UTF-8'
                },
                body: JSON.stringify({ "SessionId": this.sessionID, token, cvv })
            }

            return await (await fetch(`${this.tokenAPIURL}/Bind`, options)).json();

        } catch (error) {
            throw new TunaErrorHandler(error);
        }
    }

    async delete(token) {
        try {
            const options = {
                method: 'post',
                headers: {
                    'Content-type': 'application/json; charset=UTF-8'
                },
                body: JSON.stringify({ "SessionId": this.sessionID, token })
            }

            return await (await fetch(`${this.tokenAPIURL}/Delete`, options)).json();

        } catch (error) {
            throw new TunaErrorHandler(error);
        }
    }

    async list() {
        try {
            const options = {
                method: 'post',
                headers: {
                    'Content-type': 'application/json; charset=UTF-8'
                },
                body: JSON.stringify({ "SessionId": this.sessionID })
            }

            return await (await fetch(`${this.tokenAPIURL}/List`, options)).json();

        } catch (error) {
            throw new TunaErrorHandler(error);
        }
    }

    async generate(cardData) {
        let preparedCardData = this.filterGenerateObject(cardData);
        const options = {
            method: 'post',
            headers: {
                'Content-type': 'application/json; charset=UTF-8'
            },
            body: JSON.stringify({ "SessionId": this.sessionID, card: preparedCardData })
        }

        return await (await fetch(`${this.tokenAPIURL}/Generate`, options)).json();
    }

    isGenerateFieldValid(data, propertyName) {
        if (data == null || (typeof data === "string" && !data.trim()))
            return false;

        if (propertyName === "cardNumber") {
            if (!cardValidator.number(data).isPotentiallyValid)
                return false;
        } else if (propertyName === "expirationMonth") {
            if (data <= 0 || data > 12)
                return false;
        } else if (propertyName === "expirationYear") {
            let thisYear = new Date().getFullYear();
            if (data < thisYear || data > thisYear + 13)
                return false;
        } else if (propertyName === "singleUse") {
            if (typeof data !== "boolean")
                return false;
        }
        return true;
    }

    filterGenerateObject(creditCardData) {
        let mandatoryPropetyNames = ["cardHolderName", "cardNumber", "expirationMonth", "expirationYear", "singleUse", "cvv"];
        let notFilledProperty = [];
        let invalidFields = [];
        let tokenObject = {};

        mandatoryPropetyNames.forEach(propName => {
            if (creditCardData.hasOwnProperty(propName)) {
                if (this.isGenerateFieldValid(creditCardData[propName], propName))
                    tokenObject[propName] = creditCardData[propName];
                else
                    invalidFields.push(propName);
            }
            else
                notFilledProperty.push(propName);
        });

        if (notFilledProperty.length > 0)
            throw new TunaErrorHandler("ERR:08", { notFilledFields: notFilledProperty.join(",") });

        if (invalidFields.length > 0)
            throw new TunaErrorHandler("ERR:07", { fieldNames: invalidFields.join(",") });

        return tokenObject;
    }
}