import TunaErrorHandler from "./infra/tunaErrorHandler.js";
import { Essentials } from "./essentials.js";
import PieceManager from "./dom/pieceManager.js";
import { BuyerDocumentFormatter, BrazilianDocumentFormatter } from "./infra/buyerDocumentFormatter.js";
import GooglePayResolver from "./infra/googlePayResolver.js"

class Tuna extends Essentials {

  constructor(sessionID, env) {
    super(sessionID, env);
    this.isCheckoutPerforming = false;
    this.pieceManager = new PieceManager();
  }

  useGooglePay(googlePayBtnContainerSelector, googlePaySettings, checkoutCallbackFunction, validationGroup) {
    if (!("cardAuthMethods" in googlePaySettings)
      || !("cardNetworks" in googlePaySettings)
      || !("gatewayParameters" in googlePaySettings))
      throw new TunaErrorHandler("ERR:19");

    if (!checkoutCallbackFunction || typeof checkoutCallbackFunction !== "function")
      throw new TunaErrorHandler("ERR:09");

    if (document.querySelector(googlePayBtnContainerSelector)) {

      this.googlePayResolver = new GooglePayResolver(
        googlePaySettings,
        _ => this.onGooglePayButtonClicked(validationGroup),
        checkoutCallbackFunction);

      this.googlePayResolver.init(googlePayBtnContainerSelector);
    }
    else
      throw new TunaErrorHandler("ERR:18", { selector: googlePayBtnContainerSelector });
  }

  onGooglePayButtonClicked(validationGroup) {
    let validationGroupArray = [];
    if (validationGroup) {
      if (typeof validationGroup === "string")
        validationGroupArray = [validationGroup];
      else if (Array.isArray(validationGroup))
        validationGroupArray = validationGroup
    }

    this.isCheckoutPerforming = true;
    try {
      let screenFields;
      if (validationGroupArray.length > 0) {
        screenFields = this.mountTokenizationObject(validationGroupArray);
      }

      setTimeout(_ => this.isCheckoutPerforming = false, 500);
    } catch (e) {
      this.isCheckoutPerforming = false;
      throw e;
    }

  }

  pieceManager() {
    return this.pieceManager;
  }

  clearSavedCardSelector() {
    this.pieceManager.executeOnPieces(piece => {
      if (piece.getType() === this.pieceManager.SAVED_CARD_LIST)
        piece.clear();
    });
  }

  forgeDefaultForm(place, options) {

    if (!options || !options.document || !options.document.buyerDocumentFormatter)
      throw new TunaErrorHandler("ERR:11");

    if (!options.checkoutCallback && !options.checkoutAndPayConfig)
      throw new TunaErrorHandler("ERR:25");

    if (options.checkoutAndPayConfig && typeof options.checkoutAndPayConfig !== "object")
      throw new TunaErrorHandler("ERR:26");

    if (options.checkoutAndPayConfig) {
      try {
        if (typeof options.checkoutAndPayConfig.totalPaymentAmount !== "number" || options.checkoutAndPayConfig.totalPaymentAmount <= 0)
          throw new TunaErrorHandler("ERR:27");
        if (typeof options.checkoutAndPayConfig.paymentMethodAmount !== "number" || options.checkoutAndPayConfig.paymentMethodAmount <= 0)
          throw new TunaErrorHandler("ERR:28");
        if (typeof options.checkoutAndPayConfig.callbackFunction !== "function")
          throw new TunaErrorHandler("ERR:29");
      } catch {
        throw new TunaErrorHandler("ERR:30");
      }
    }

    this.pieceManager.forge(place, "defaultForm",
      {
        interfaceManager: this,
        pieceManager: this.pieceManager,
        options: options || {}
      });

  }

  useQrCodePayment(place, options, paymentCallback, group = "default") {

    if (!paymentCallback || typeof paymentCallback !== "function")
      throw new TunaErrorHandler("ERR:24");

    if (!options || options.methodID === "undefined" || typeof options.methodID !== "number")
      throw new TunaErrorHandler("ERR:32", { propName: "methodID" });

    if (!options.paymentKey || typeof options.paymentKey !== "string")
      throw new TunaErrorHandler("ERR:32", { propName: "paymentKey" });

    this.pieceManager.forge(place, "QrCodePaymentBox", options, group);

    this.doStatusLongPolling(paymentCallback, options.methodID, options.paymentKey);
  }

  async useSavedCardSelector(place, options, group = "default") {
    let skeletonID;
    skeletonID = this.pieceManager.forge(place, "savedCardSkeleton");

    let response = await this.tokenizator().list();
    if (response && response.code === 1 && response.tokens && response.tokens.length > 0) {
      if (skeletonID)
        this.pieceManager.destroy(skeletonID);

      this.pieceManager.forge(place, "savedCardSelector", {
        tokens: response.tokens,
        pieceManager: this.pieceManager,
        ...options
      }, group);

      return response.tokens.length;
    } else {
      if (skeletonID)
        this.pieceManager.destroy(skeletonID);
      return 0;
    }
  }

  async checkoutAndPay(paymentMethodType, totalPaymentAmount, paymentMethodAmount, callbackFunction) {

    if (this.isCheckoutPerforming)
      throw new TunaErrorHandler("ERR:14");

    let isCallbackCheckout = callbackFunction && typeof callbackFunction === "function";
    let prepearedCardData;

    try {
      this.isCheckoutPerforming = true;
      if (paymentMethodType === "creditCard")
        prepearedCardData = await this.prepareCheckout();

    } catch (e) {

      this.isCheckoutPerforming = false;

      if (isCallbackCheckout)
        callbackFunction(new Promise((resolve, reject) => reject(e)), null);
      else
        throw e;
    }

    setTimeout(_ => this.isCheckoutPerforming = false, 500);

    if (isCallbackCheckout)
      callbackFunction(this.pay(paymentMethodType, totalPaymentAmount, paymentMethodAmount, prepearedCardData));
    else
      return await this.pay(paymentMethodType, totalPaymentAmount, paymentMethodAmount, prepearedCardData);
  }

  async prepareCheckout() {

    let screenFields = this.mountTokenizationObject();

    if (!screenFields)
      throw new TunaErrorHandler("ERR:16");

    else if (Object.keys(screenFields).length === 0 && screenFields.constructor === Object)
      throw new TunaErrorHandler("ERR:15");

    let savedCardData = await this.getSavedCardData(screenFields);

    if (savedCardData !== null)
      return {
        fullCardInfo: savedCardData,
        alreadyTokenized: true
      };

    return {
      cardData: screenFields,
      alreadyTokenized: false
    };
  }

  async checkout(callbackFunction) {
    if (this.isCheckoutPerforming)
      throw new TunaErrorHandler("ERR:14");

    let isCallbackCheckout = callbackFunction && typeof callbackFunction === "function";

    try {
      this.isCheckoutPerforming = true;

      let prepearedCheckout = await this.prepareCheckout();

      setTimeout(_ => this.isCheckoutPerforming = false, 500);

      if (prepearedCheckout.alreadyTokenized) {

        let fullCardinfo = prepearedCheckout.fullCardInfo;

        if (isCallbackCheckout) {
          callbackFunction(
            new Promise((resolve) => {
              resolve({ ...fullCardinfo.tokenData, code: 1 });
            }), fullCardinfo.cardData);
        } else {
          return {
            success: true,
            tokenData: { ...fullCardinfo.tokenData, code: 1 },
            cardData: fullCardinfo.cardData
          };
        }
      } else {
        if (isCallbackCheckout)
          this.tokenize(prepearedCheckout.cardData, callbackFunction);
        else
          return await this.tokenize(prepearedCheckout.cardData);
      }

    } catch (e) {
        this.isCheckoutPerforming = false;

      if (isCallbackCheckout)
        callbackFunction(new Promise((resolve, reject) => reject(e)), null);
      else
        throw e;
    }
  }
  //DEPRECATED
  async directCheckout() {
    return await this.checkout();
  }

  async getSavedCardData(screenFields) {
    if (screenFields && screenFields.savedCard != null) {
      let savedCard = screenFields.savedCard;

      let bindedToken = await this.tokenizator().bind(savedCard.token, savedCard.cvv);

      if (!bindedToken || bindedToken.code !== 1)
        throw new TunaErrorHandler("ERR:17");

      let tokenData = { brand: savedCard.brand, token: savedCard.token };

      delete savedCard.brand;
      delete savedCard.token;
      delete savedCard.cvv;

      delete screenFields.savedCard
      delete screenFields.cvv;

      return { tokenData, cardData: { ...screenFields, ...savedCard } };
    } else return null;
  }

  mountTokenizationObject(validationGroupArray = null) {
    let response = {};
    let invalidFieldPropertyNames = [];
    this.pieceManager.executeOnPieces(piece => {

      if (validationGroupArray === null || (Array.isArray(validationGroupArray) && validationGroupArray.includes(piece.getGroup()))) {
        let pieceType = piece.getType();
        if (pieceType.propertyName && piece.isEnabled()) {
          try {
            let value = piece.getValue();

            if (pieceType.propertyName === "expiration" && value) {
              response = {
                ...response,
                "expirationYear": value.expirationYear,
                "expirationMonth": value.expirationMonth
              };
            } else
              response = { ...response, [pieceType.propertyName]: value };

          } catch {
            invalidFieldPropertyNames.push(pieceType.propertyName);
          }
        } else if (!piece.isEnabled()) {
          piece.resetInvalidMarkup();
        }
      }
    });
    if (invalidFieldPropertyNames.length > 0)
      throw new TunaErrorHandler("ERR:07", { fieldNames: invalidFieldPropertyNames.join(", ") });

    if (response.singleUse !== undefined) {
      // At screen, the common is use "save card" checkbox. Once the tokenization expects a "single use" flag, the value need to be inverted
      response.singleUse = !response.singleUse;
    }

    return response;
  }

  getBuyerDocumentFormatter(locale) {
    switch (locale) {
      case "pt-BR":
        return new BrazilianDocumentFormatter();
      default:
        return new BuyerDocumentFormatter();
    }
  }
}

export default function factory(sessionID, env = "production") {
  if (sessionID)
    return new Tuna(sessionID, env);
  else
    throw new TunaErrorHandler("ERR:34");
}